/* 
EasyDriver stepper test sketch
 
Circuit diagram and description at http://elabz.com/manually-controlling-bipolar-stepper-motor-with-arduino-and-easydriver/

 */

// constants won't change. They're used here to 
// set pin numbers:
const byte stepUpPin = 4;     // the number of the step pin
const byte stepDownPin = 5;     // the number of the step pin
const byte stepOut = 2;
const byte directionOut = 3;
const byte ledPin =  12;      // pin # for direction indicator LED 
const byte readyPin =  11;      //pin # for LED that comes on when all the steps have been completed
const byte inMotionPin = 10; // pin # for the LED that light up when the stepper is supposed to be still moving
const byte ratePin = 0; // A0 - analog 0 pin on Arduino to control the stepping dealy (i.e. RPMs)
const byte enablePin = 6; // turn EasyDriver off when not turning (saves power but ONLY use when there's no need to hold position)

// Variables will change:
byte ledState = LOW;         // the current state of the output pin
byte lastStepUpButtonState = HIGH;   // the previous reading from the step UP pin
byte lastStepDownButtonState = HIGH;   // the previous reading from the step DOWN pin
byte directionState=HIGH;             // the current direction
byte stepUpState=HIGH;             // the current state of UP button 
byte stepDownState=HIGH;             // the current state of DOWN button
int stepsPassed = 0; //how many steps?
int stepsPassedMax = 160; // trying to calculate steps per revolution (remember the microstepping settings 1/8th is default)
// most small and micro steppers, especially those that came from a CD- or DVD-R/RW drives 
// have 20 steps per revolution. So 160 mircosteps should make the motor spin 360 degrees once.

long lastStepUpDebounceTime = 0;  // the last time the output pin was toggled
long lastStepDownDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 50;    // the debounce time in ms; increase if the output flickers

long stepDelayBase = 1; // 1ms base rate. Multiply by the reading from the RATE potentiometer for actual step delay
long stepDelay; // 
long lastStepTime = 0; // last time the step signal was sent

void setup() {
  pinMode(stepUpPin, INPUT);
  pinMode(stepDownPin, INPUT);
  pinMode(ratePin, INPUT); 
  pinMode(stepOut, OUTPUT);  
  pinMode(directionOut, OUTPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(readyPin, OUTPUT);
  pinMode(inMotionPin, OUTPUT);
  pinMode(enablePin, OUTPUT);
  digitalWrite(readyPin, HIGH); // turn OFF all LEDs in the beginning
  digitalWrite(inMotionPin, HIGH);  // turn OFF all LEDs in the beginning
  digitalWrite(ledPin, HIGH); // turn OFF all LEDs in the beginning
}

void loop() {

// read the state of the switch into a local variable:
int readingStepUp = digitalRead(stepUpPin);
int readingStepDown = digitalRead(stepDownPin);
stepDelay = analogRead(ratePin) * stepDelayBase/50;
if(stepDelay < 1) stepDelay = 1; // reality check - a pot can read 0 and then it would mean infinite RMP - not possible

if(readingStepUp == LOW || readingStepDown == LOW) { // only read buttons if either one of them is LOW

  // If the switch changed, due to noise or pressing:
  if (readingStepUp != lastStepUpButtonState) {
    // reset the debouncing timer
    lastStepUpDebounceTime = millis();
    lastStepUpButtonState = readingStepUp;
  } 
  
  if ((millis() - lastStepUpDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:
    lastStepUpButtonState = readingStepUp;
    lastStepUpDebounceTime = millis();
    stepUpState = readingStepUp;
  }



  // If the switch changed, due to noise or pressing:
  if (readingStepDown != lastStepDownButtonState) {
    // reset the debouncing timer
    lastStepDownDebounceTime = millis();
    lastStepDownButtonState = readingStepDown;
  } 
  
  if ((millis() - lastStepDownDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:
    lastStepDownButtonState = readingStepDown;
    lastStepDownDebounceTime = millis();    
    stepDownState = readingStepDown;

  }

  
}else{
 stepUpState = HIGH;
 stepDownState = HIGH;
 
} // end of if block that reads button states
  
if(stepsPassed == 0 ) { // if the previous command has completed, make the direction decision 
    if(stepUpState == LOW || stepDownState == LOW) { 
      if(stepUpState == LOW && stepDownState == LOW) { directionState = LOW; } // why are you holding both UP and DOWN buttons?
      if(stepUpState == LOW && stepDownState == HIGH) { directionState = HIGH;} // go up
      if(stepUpState == HIGH && stepDownState == LOW) { directionState = LOW;} // go down
      stepsPassed = stepsPassedMax+1;
  }
 } 
  

if(stepsPassed > 0 ) // send step signals now
{
  digitalWrite(enablePin, LOW); // wake up
  digitalWrite(ledPin, directionState); // set proper direction
  digitalWrite(directionOut, directionState); 
  if((millis() - lastStepTime) > stepDelay) // delay expired, send another step
    {
        digitalWrite(stepOut, HIGH);
        stepsPassed--;
        lastStepTime = millis();
    }else
    {
      // time for one step not yet expired, hold the STEP signal low 
      digitalWrite(stepOut, LOW);
    }
}else{
 digitalWrite(enablePin, HIGH); // go to sleep

}

if(stepsPassed == 0) { 
 digitalWrite(readyPin, LOW);
 digitalWrite(inMotionPin, HIGH); 

 }else{
 digitalWrite(readyPin, HIGH);
 digitalWrite(inMotionPin, LOW); 
 }

}

